
#include <math.h>
#include <camera.h>

static const double aspect = 4.0 / 3.0;

Camera::Camera( const Point3 &pos,
                const Vector3 &upv,
                const Vector3 &fwdv,
                const double &initfov )
    : eye( pos ), up( upv ), forward( fwdv ), fov( initfov )
{
    /* Set camera position. */
    viewtoworld = buildObjectToWorld( eye, forward, up );

    /**
     *  fov     Pi 
     *  ---  *  ---
     *   2      180
     *            ^--- get into radians
     *   ^-- since we only want the top half thingy (want a right triangle)
     */
    screendist = 0.5 / tan( ( 0.5 * fov * M_PI ) / 180.0 );
}

Camera::~Camera( void )
{
}

void Camera::transformEyePosition( const Matrix4x4 &xform )
{
    eye = multiplyPointByMatrix( xform, eye );
    viewtoworld = buildObjectToWorld( eye, forward, up );
}

Vector3 Camera::screenRay( double x, double y ) const
{
    /* The point we're sampling in view coordinates. */
    Point3 destinview( ( x - 0.5 ) * aspect, ( 0.5 - y ), screendist );

    /* Calculate the ray direction from the eye in world space. */
    Point3 raydest = multiplyPointByMatrix( viewtoworld, destinview );

    /* XXX: Should I normalize?? */
    return raydest.subtract( eye ).normalize();
}

