
#ifndef COLOUR_H_INCLUDED
#define COLOUR_H_INCLUDED

#ifdef __cplusplus
extern "C" {
#endif

/**
 * C utilities for manipulating colour.
 */
double linear_to_nonlinear_rgb( double val );
double nonlinear_to_linear_rgb( double val );

/**
 * Convert linear RGB to CIE XYZ in [0,1].
 */
void rgb_to_xyz( double  r, double  g, double  b,
                 double *x, double *y, double *z );

/**
 * Convert CIE XYZ to linear RGB in [0,1].
 */
void xyz_to_rgb( double  x, double  y, double  z,
                 double *r, double *g, double *b );

/**
 * Convert linear RGB to fake-L*a*b* for interpolations.
 */
void rgb_to_lab( double r, double g, double b,
                 double *lstar, double *astar, double *bstar );

/**
 * Convert fake-L*a*b* back to linear RGB.
 */
void lab_to_rgb( double lstar, double astar, double bstar,
                 double *r, double *g, double *b );

#ifdef __cplusplus
};
#endif

#endif /* COLOUR_H_INCLUDED */
