
#ifndef CONVEXPOLY_H_INCLUDED
#define CONVEXPOLY_H_INCLUDED

#include <primitive.h>
#include <vertexlist.h>

/**
 * Polygon intersection code from:
 *
 * Haines, Eric, "Point in Polygon Strategies," Graphics Gems IV, ed.
 * Paul Heckbert, Academic Press, p. 24-46, 1994
 *
 * My reference is:
 * http://www.acm.org/pubs/tog/editors/erich/ptinpoly/
 */

class ConvexPoly : public Primitive
{
public:
    explicit ConvexPoly( VertexList *vlist );
    ~ConvexPoly( void );

    BoundingBox getBoundingBox( void ) const;
    PrimitiveList foldModel( const Matrix4x4 &xform ) const;

    bool closestIntersect( const Point3 &origin, const Vector3 &dir,
                           double *t, Vector3 *norm, Material **mat ) const;

private:
    Point3 *ptlist;
    int numpts;
    double a, b, c, d;
    bool bigx, bigy, bigz;
    void *pset;

    bool planeIntersect( const Point3 &origin, const Vector3 &dir,
                         double *t, Vector3 *norm ) const;
};

#endif // CONVEXPOLY_H_INCLUDED
