
#include <stdio.h>
#include <sys/types.h>
#include <unistd.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <vector3.h>
#include <scene.h>
#include <shader.h>
#include <sceneparser.h>
#include <pngoutput.h>
#include <colour.h>

void outputRgb24Scanline( unsigned char *output, PixelXYZ *scanline, int width )
{
    while( width-- ) {
        double r, g, b;

        xyz_to_rgb( scanline->getX(), scanline->getY(), scanline->getZ(),
                    &r, &g, &b );

        if( r > 1.0 ) r = 1.0; if( r < 0.0 ) r = 0.0;
        if( g > 1.0 ) g = 1.0; if( g < 0.0 ) g = 0.0;
        if( b > 1.0 ) b = 1.0; if( b < 0.0 ) b = 0.0;

        r = ( linear_to_nonlinear_rgb( r ) * 255.0 ) + 0.5;
        g = ( linear_to_nonlinear_rgb( g ) * 255.0 ) + 0.5;
        b = ( linear_to_nonlinear_rgb( b ) * 255.0 ) + 0.5;

        *output++ = (unsigned char) r;
        *output++ = (unsigned char) g;
        *output++ = (unsigned char) b;
        scanline++;
    }
}

int main( int argc, char **argv )
{
    if( argc < 3 ) {
        fprintf( stderr, "Usage: %s scene_desc.txt output.png\n", argv[ 0 ] );
        return 1;
    }

    // Create a parser and load in the data.
    SceneParser p( argv[ 1 ] );

    // Parse the data into a folded scene.
    Scene scene( p.getNode( "scene" ), p.getGridWidth() );
    scene.setCamera( p.getCamera() );

    // Create a shader which we'll use to sample the scene.
    Shader shader( &scene );

    // Temporary space for the images.
    PixelXYZ *image = new PixelXYZ[ p.getWidth() ];
    unsigned char *output = new unsigned char[ p.getWidth() * 3 ];
    if( !image || !output ) {
        fprintf( stderr, "portray: Not enough memory for image.\n" );
        return 1;
    }

    // Output to PNG.
    pngoutput_t *pngoutput = pngoutput_new( argv[ 2 ],
                                            p.getWidth(), p.getHeight(),
                                            0.45 );

    // Render each scanline using the desired sampling method.
    for( int y = 0; y < p.getHeight(); y++ ) {
        fprintf( stderr, "portray: Raytracing scanline %3d.\r", y );
        //scene.adaptiveSampleScanline( image, y, p.getWidth(), p.getHeight(), 2 );
        //shader.superSampleScanline( image, y, p.getWidth(), p.getHeight(), 2 );
        shader.pointSampleScanline( image, y, p.getWidth(), p.getHeight() );
        outputRgb24Scanline( output, image, p.getWidth() );
        pngoutput_scanline( pngoutput, output );
    }
    fprintf( stderr, "\nportray: Raytracing complete.\n" );

    // Clean up.
    pngoutput_delete( pngoutput );
    delete[] image;
    delete[] output;
    return 0;
}

