
#ifndef MATRIX4X4_H_INCLUDED
#define MATRIX4X4_H_INCLUDED

#include <point3.h>
#include <vector3.h>

/**
 *      0  1  2  3
 * 0 [  0  1  2  3 ]
 * 1 [  4  5  6  7 ]
 * 2 [  8  9 10 11 ]
 * 3 [ 12 13 14 15 ]
 */

class Matrix4x4
{
public:
    double d[ 4 ][ 4 ];
};

/**
 * Matrix by vector.
 */
Vector3 multiplyVectorByMatrix( const Matrix4x4 &m, const Vector3 &v );

/**
 * Matrix by point.
 */
Point3 multiplyPointByMatrix( const Matrix4x4 &m, const Point3 &v );

/**
 * Matrix by matrix.
 */
Matrix4x4 multiplyMatrix( const Matrix4x4 &m1, const Matrix4x4 &m2 );

/**
 * Conversion between basis.
 */
Matrix4x4 buildObjectToWorld( const Point3 &position,
                              const Vector3 &forward,
                              const Vector3 &up );

/**
 * Translation.
 */
Matrix4x4 buildTranslation( const double &x, const double &y, const double &z );

/**
 * Scale about the origin.
 */
Matrix4x4 buildScale( const double &sx, const double &sy, const double &sz );

/**
 * Rotate about the X axis;
 */
Matrix4x4 buildRotationX( const double &angle );

/**
 * Rotate about the Y axis;
 */
Matrix4x4 buildRotationY( const double &angle );

/**
 * Rotate about the Z axis;
 */
Matrix4x4 buildRotationZ( const double &angle );

/**
 * Transpose the given matrix.
 */
Matrix4x4 transposeMatrix( const Matrix4x4 &m );

/**
 * Returns the inverse of the given matrix.
 */
Matrix4x4 inverseMatrix( const Matrix4x4 &m );

/**
 * Returns the identity matrix.
 */
Matrix4x4 identityMatrix( void );

#endif // MATRIX4X4_H_INCLUDED
