
#ifndef NODE_H_INCLUDED
#define NODE_H_INCLUDED

#include <stl.h>
#include <string>
#include <vector>
#include <baseobject.h>
#include <matrix4x4.h>

/**
 * A node wraps a base object with a transformation matrix.  This
 * will delete the given object when it's done.
 */
class Node : public BaseObject
{
public:
    Node( void );
    ~Node( void );

    void setName( const char *name ) { nodename = name; }
    const char *getName( void ) const { return nodename.c_str(); }

    PrimitiveList foldModel( const Matrix4x4 &xform ) const;

    void applyMatrix( const Matrix4x4 &xform );

    Matrix4x4 getTransform( void ) const { return forward; }

    void addObject( BaseObject *object ) { objects.push_back( object ); }
    void setMaterial( Material *m, bool override = true );
    bool closestIntersect( const Point3 &origin, const Vector3 &dir,
                           double *t, Vector3 *norm, Material **mat ) const;

private:
    vector<BaseObject *> objects;
    Matrix4x4 forward;
    Matrix4x4 reverse;
    std::string nodename;
};

#endif // NODE_H_INCLUDED
