

#ifndef PIXELXYZ_H_INCLUDED
#define PIXELXYZ_H_INCLUDED

#include <colour.h>

class PixelXYZ
{
public:
    PixelXYZ( void )
        : x( 0.0 ), y( 0.0 ), z( 0.0 ) {}
    PixelXYZ( const double &xinit, const double &yinit, const double &zinit )
        : x( xinit ), y( yinit ), z( zinit ) {}
    ~PixelXYZ( void ) {}

    static PixelXYZ black;

    PixelXYZ multiply( const PixelXYZ &right ) const
    {
        return PixelXYZ( x * right.x, y * right.y, z * right.z );
    }

    PixelXYZ add( const PixelXYZ &right ) const
    {
        return PixelXYZ( x + right.x, y + right.y, z + right.z );
    }

    PixelXYZ subtract( const PixelXYZ &right ) const
    {
        return PixelXYZ( x - right.x, y - right.y, z - right.z );
    }

    PixelXYZ scalarMult( double val ) const
    {
        return PixelXYZ( x * val, y * val, z * val );
    }

    double intensity( void ) const
    {
        return sqrt( ( x * x ) + ( y * y ) + ( z * z ) );
    }

    double intensitySquared( void ) const
    {
        return ( x * x ) + ( y * y ) + ( z * z );
    }

    double getX( void ) const { return x; }
    double getY( void ) const { return y; }
    double getZ( void ) const { return z; }

    static PixelXYZ createPixelRGB( const double &r, const double &g,
                                    const double &b )
    {
        double x, y, z;
        rgb_to_xyz( r, g, b, &x, &y, &z );
        return PixelXYZ( x, y, z );
    }

private:
    double x, y, z;
};

#endif // PIXELXYZ_H_INCLUDED
