
#ifndef SCENE_H_INCLUDED
#define SCENE_H_INCLUDED

#include <stl.h>
#include <vector>
#include <point3.h>
#include <vector3.h>
#include <material.h>
#include <matrix4x4.h>
#include <baseobject.h>
#include <uniformgrid.h>
#include <camera.h>

class Scene
{
public:
    Scene( BaseObject *scene, int gridwidth );
    ~Scene( void );

    void setCamera( Camera *newc );
    Camera *getCamera( void ) const { return c; }

    const PrimitiveList &getLights( void ) const { return lights; }

    bool closestIntersect( const Point3 &origin, const Vector3 &dir,
                           double *t, Vector3 *norm, Material **mat ) const;

private:
    UniformGrid grid;
    PrimitiveList objects;
    PrimitiveList lights;
    Camera *c;
    bool ugrid;
    double gridscale;
    int gridw;

    void addScene( BaseObject *root );
};

#endif // SCENE_H_INCLUDED
