
#include <math.h>
#include <stdlib.h>
#include <node.h>
#include <sphere.h>

Sphere::Sphere( void )
{
    forward = identityMatrix();
    reverse = identityMatrix();
}

Sphere::Sphere( const Matrix4x4 &xform )
{
    forward = xform;
    reverse = inverseMatrix( forward );
}

Sphere::~Sphere( void )
{
}

BoundingBox Sphere::getBoundingBox( void ) const
{
    BoundingBox b;
    Point3 p;

    b.minx = HUGE_VAL; b.maxx = -HUGE_VAL;
    b.miny = HUGE_VAL; b.maxy = -HUGE_VAL;
    b.minz = HUGE_VAL; b.maxz = -HUGE_VAL;

    p = multiplyPointByMatrix( forward, Point3( 1.0, 1.0, 1.0 ) );
    if( p.getX() < b.minx ) b.minx = p.getX();
    if( p.getX() > b.maxx ) b.maxx = p.getX();
    if( p.getY() < b.miny ) b.miny = p.getY();
    if( p.getY() > b.maxy ) b.maxy = p.getY();
    if( p.getZ() < b.minz ) b.minz = p.getZ();
    if( p.getZ() > b.maxz ) b.maxz = p.getZ();

    p = multiplyPointByMatrix( forward, Point3( 1.0, 1.0, -1.0 ) );
    if( p.getX() < b.minx ) b.minx = p.getX();
    if( p.getX() > b.maxx ) b.maxx = p.getX();
    if( p.getY() < b.miny ) b.miny = p.getY();
    if( p.getY() > b.maxy ) b.maxy = p.getY();
    if( p.getZ() < b.minz ) b.minz = p.getZ();
    if( p.getZ() > b.maxz ) b.maxz = p.getZ();

    p = multiplyPointByMatrix( forward, Point3( 1.0, -1.0, 1.0 ) );
    if( p.getX() < b.minx ) b.minx = p.getX();
    if( p.getX() > b.maxx ) b.maxx = p.getX();
    if( p.getY() < b.miny ) b.miny = p.getY();
    if( p.getY() > b.maxy ) b.maxy = p.getY();
    if( p.getZ() < b.minz ) b.minz = p.getZ();
    if( p.getZ() > b.maxz ) b.maxz = p.getZ();

    p = multiplyPointByMatrix( forward, Point3( -1.0, 1.0, 1.0 ) );
    if( p.getX() < b.minx ) b.minx = p.getX();
    if( p.getX() > b.maxx ) b.maxx = p.getX();
    if( p.getY() < b.miny ) b.miny = p.getY();
    if( p.getY() > b.maxy ) b.maxy = p.getY();
    if( p.getZ() < b.minz ) b.minz = p.getZ();
    if( p.getZ() > b.maxz ) b.maxz = p.getZ();

    p = multiplyPointByMatrix( forward, Point3( -1.0, -1.0, 1.0 ) );
    if( p.getX() < b.minx ) b.minx = p.getX();
    if( p.getX() > b.maxx ) b.maxx = p.getX();
    if( p.getY() < b.miny ) b.miny = p.getY();
    if( p.getY() > b.maxy ) b.maxy = p.getY();
    if( p.getZ() < b.minz ) b.minz = p.getZ();
    if( p.getZ() > b.maxz ) b.maxz = p.getZ();

    p = multiplyPointByMatrix( forward, Point3( 1.0, -1.0, -1.0 ) );
    if( p.getX() < b.minx ) b.minx = p.getX();
    if( p.getX() > b.maxx ) b.maxx = p.getX();
    if( p.getY() < b.miny ) b.miny = p.getY();
    if( p.getY() > b.maxy ) b.maxy = p.getY();
    if( p.getZ() < b.minz ) b.minz = p.getZ();
    if( p.getZ() > b.maxz ) b.maxz = p.getZ();

    p = multiplyPointByMatrix( forward, Point3( -1.0, 1.0, -1.0 ) );
    if( p.getX() < b.minx ) b.minx = p.getX();
    if( p.getX() > b.maxx ) b.maxx = p.getX();
    if( p.getY() < b.miny ) b.miny = p.getY();
    if( p.getY() > b.maxy ) b.maxy = p.getY();
    if( p.getZ() < b.minz ) b.minz = p.getZ();
    if( p.getZ() > b.maxz ) b.maxz = p.getZ();

    p = multiplyPointByMatrix( forward, Point3( -1.0, -1.0, -1.0 ) );
    if( p.getX() < b.minx ) b.minx = p.getX();
    if( p.getX() > b.maxx ) b.maxx = p.getX();
    if( p.getY() < b.miny ) b.miny = p.getY();
    if( p.getY() > b.maxy ) b.maxy = p.getY();
    if( p.getZ() < b.minz ) b.minz = p.getZ();
    if( p.getZ() > b.maxz ) b.maxz = p.getZ();

    return b;
}

PrimitiveList Sphere::foldModel( const Matrix4x4 &xform ) const
{
    PrimitiveList pl;
    Sphere *s = new Sphere( xform );
    s->setMaterial( getMaterial() );
    pl.push_back( s );
    return pl;
}

static inline bool quadraticRoots( double &root1, double &root2,
                                   const double &a, const double &b, const double &c )
{
    /**
     * -b +/- sqrt( b^2 - 4ac )
     * -----------------------
     *          2a
     */
    double det = ( b * b ) - ( 4.0 * a * c );
    if( det < 0.0 ) return false;

    double bot = ( 2.0 * a );
    double sqdet = sqrt( det );
    root1 = ( -b + sqdet ) / bot;
    root2 = ( -b - sqdet ) / bot;
    return true;
}

bool Sphere::closestIntersect( const Point3 &origin, const Vector3 &dir,
                               double *t, Vector3 *normal, Material **mat ) const
{
    Point3 neworigin = multiplyPointByMatrix( reverse, origin );
    Vector3 newdir = multiplyVectorByMatrix( reverse, dir );

    Vector3 diff( neworigin.subtract( Point3::origin ) );
    double root1, root2;

    if( quadraticRoots( root1, root2,
                        newdir.lengthSquared(),
                        2.0 * newdir.dotProduct( diff ),
                        neworigin.subtract( Point3::origin ).lengthSquared() - 1.0 ) ) {

        if( root1 < 0.0 && root2 < 0.0 ) return false;

        double nearest = root1;

        if( root1 < 0.0 ) {
            nearest = root2;
        } else if( root2 < 0.0 ) {
            nearest = root1;
        } else if( root2 < nearest ) {
            nearest = root2;
        }

        *t = nearest;
        Vector3 newnormal = neworigin.add( newdir.scalarMult( nearest ) ).subtract( Point3::origin );
        *normal = multiplyVectorByMatrix( forward, newnormal );
        *mat = getMaterial();

        return true;
    } else {
        return false;
    }
}

