
#ifndef SPHERE_H_INCLUDED
#define SPHERE_H_INCLUDED

#include <math.h>
#include <primitive.h>

/**
 * The unit sphere.
 */
class Sphere : public Primitive
{
public:
    Sphere( void );
    explicit Sphere( const Matrix4x4 &xform );
    virtual ~Sphere( void );

    BoundingBox getBoundingBox( void ) const;

    PrimitiveList foldModel( const Matrix4x4 &xform ) const;

    /**
     * Returns a unit vector for a random point on the sphere.
     */
    static Vector3 uniformVectorFromSphere( void )
    {
        double z = ( 2.0 * ( (double) rand() / (double) RAND_MAX ) ) - 1.0;
        double theta = ( (double) rand() * 2.0 * M_PI ) / (double) RAND_MAX;

        /* Convert from cylinder coordinates. */
        double r = sqrt( 1.0 - ( z * z ) );
        return Vector3( r * cos( theta ), r * sin( theta ), z );
    }

    Point3 getRandomLightPosition( const Point3 &closest ) const
    {
        Vector3 tempnorm = closest.subtract( Point3::origin ).normalize();
        Vector3 uniform = Sphere::uniformVectorFromSphere();

        if( uniform.dotProduct( tempnorm ) < 0.0 ) {
            uniform = uniform.scalarMult( -1.0 );
        }

        return Point3::origin.add( uniform );
    }

    bool closestIntersect( const Point3 &origin, const Vector3 &dir,
                           double *t, Vector3 *normal, Material **mat ) const;

private:
    Matrix4x4 forward;
    Matrix4x4 reverse;
};

#endif // SPHERE_H_INCLUDED
